package Yaala::Parser;

use strict;
use warnings;
use vars qw#%DATAFIELDS#;

use Exporter;
use Yaala::Data::Persistent qw#init#;
use Yaala::Parser::WebserverTools qw#%MONTH_NUMBERS#;

@Yaala::Parser::EXPORT_OK = qw(parse extra %DATAFIELDS);
@Yaala::Parser::ISA = ('Exporter');

our $LASTDATE = init ('$LASTDATE', 'scalar');
                
if (!$$LASTDATE) { $$LASTDATE = 0; }

%DATAFIELDS =
(
	source_ip	=> 'key',
	dest_ip		=> 'key',
	
	incoming_dev	=> 'key',
	outgoing_dev	=> 'key',

	protocol	=> 'key',

	source_port	=> 'key:numeric',
	destination_port => 'key:numeric',

	icmp_type	=> 'key:numeric',

	mac_address	=> 'key',

	tos		=> 'key',
	prec		=> 'key',
	ttl		=> 'key:numeric',

	packets		=> 'agg',
	size		=> 'agg',
	payload		=> 'agg'
);


# This needs to be done at runtime, since Data uses Setup which relies on
# %DATAFIELDS to be defined  -octo
require Yaala::Data::Core;
import Yaala::Data::Core qw#store#;

my $VERSION = '$Id: Iptables.pm,v 1.4 2003/12/07 15:21:02 octo Exp $';
print STDERR $/, __FILE__, ": $VERSION" if ($::DEBUG);

return (1);

sub parse
{
	my $line = shift;

	if ($line =~ m/IN=\S* OUT=/)
	{
		my ($month, $day, $hour, $minute, $second) = $line =~ m/^(\w{3}) (\d+) (\d\d):(\d\d):(\d\d)/;
		my $year = (localtime ())[5] + 1900;
		$month = $MONTH_NUMBERS{$month};

		{
			my $tmp = int (sprintf ("%04u%02u%02u%02u%02u%02u",
					$year, $month, $day, $hour, $minute, $second));

			if ($tmp < $$LASTDATE)
			{
				print STDERR $/, __FILE__, ": Skipping.. ($tmp < $$LASTDATE)" if ($::DEBUG & 0x0200);
				return (undef);
			}
			else { $$LASTDATE = $tmp; }
		}
		
		my $date = sprintf ("%04u-%02u-%02u", $year, $month, $day);

		my %packet = ();
		while ($line =~ m/([A-Z]+)=(\S+)/g)
		{
			my $key = lc ($1);
			my $val = $2;

			if ($key eq 'len')
			{
				if (defined ($packet{'size'}))
				{
					$packet{'payload'} = $val;
				}
				else
				{
					$packet{'size'} = $val;
				}
			}
			else
			{
				$packet{$key} = $val;
			}
		}

		my %data =
		(
			source_ip	=> 'n/a',
			dest_ip		=> 'n/a',
			
			incoming_dev	=> '*none*',
			outgoing_dev	=> '*none*',

			protocol	=> '*unknown*',

			source_port	=> 0,
			destination_port => 0,
			icmp_type	=> 0,

			mac_address	=> '*unknown*',

			tos		=> '0x00',
			prec		=> '0x00',
			ttl		=> 0,

			packets		=> 1,
			size		=> 0,
			payload		=> 0
		);

		$data{'source_ip'} = $packet{'src'} if (defined ($packet{'src'}));
		$data{'dest_ip'} = $packet{'dst'} if (defined ($packet{'dst'}));

		$data{'incoming_dev'} = $packet{'in'} if (defined ($packet{'in'}));
		$data{'outgoing_dev'} = $packet{'out'} if (defined ($packet{'out'}));

		$data{'protocol'} = $packet{'proto'} if (defined ($packet{'proto'}));

		$data{'source_port'} = $packet{'spt'} if (defined ($packet{'spt'}));
		$data{'destination_port'} = $packet{'dpt'} if (defined ($packet{'dpt'}));
		$data{'icmp_type'} = $packet{'type'} if (defined ($packet{'type'}));

		$data{'mac_address'} = $packet{'mac'} if (defined ($packet{'mac'}));

		$data{'tos'} = $packet{'tos'} if (defined ($packet{'tos'}));
		$data{'prec'} = $packet{'prec'} if (defined ($packet{'prec'}));
		$data{'ttl'} = $packet{'ttl'} if (defined ($packet{'ttl'}));

		$data{'size'} = $packet{'size'} if (defined ($packet{'size'}));
		$data{'payload'} = $packet{'payload'} if (defined ($packet{'payload'}));
		
		store (\%data);
	}
}

sub extra
{
}
